#include "component_behavior_boomerang.h"

/* DEPENDENCIES: Physics, Health */

static void _begin(ComponentBehaviorBoomerang* self, ECS* ecs);
static void _boomerang(ComponentBehaviorBoomerang* self, ECS* ecs);

/* Resets after a boomerang is finished. */
static void
_begin(ComponentBehaviorBoomerang* self, ECS* ecs)
{
	ComponentPhysics* physics;
	ComponentPhysics* targetPhysics;
	ComponentAnimationLevitate* animationLevitate;

	physics = ecs_component_get(ecs, ECS_COMPONENT_PHYSICS, self->component.id);
	animationLevitate = ecs_component_get(ecs, ECS_COMPONENT_ANIMATION_LEVITATE, self->component.id);
	
	targetPhysics = ecs_component_get(ecs, ECS_COMPONENT_PHYSICS, self->targetID);

	/* this should not happen. */
	if (!targetPhysics)
		return;

	self->angle = ATAN
	(
		physics->position[0],
		targetPhysics->position[0],
		physics->position[1],
		targetPhysics->position[1]
	);

	/* Fling the opposite way as a heads up. */
	physics->velocity[0] = -cos(self->angle) * self->initialSpeed;
	physics->velocity[1] = -sin(self->angle) * self->initialSpeed;

	if (animationLevitate)
	{
		ComponentGameObject* gameObject;

		gameObject = ecs_component_get(ecs, ECS_COMPONENT_GAME_OBJECT, self->component.id);

		gameObject->height = animationLevitate->height;
		animationLevitate->component.isDisabled = true;
	}

	self->isBegin = true;
}

/* A c c e l e r a t e . */
static void
_boomerang(ComponentBehaviorBoomerang* self, ECS* ecs)
{
	ComponentPhysics* physics;

	physics = ecs_component_get(ecs, ECS_COMPONENT_PHYSICS, self->component.id);

	physics->velocity[0] += cos(self->angle) * self->speed;
	physics->velocity[1] += sin(self->angle) * self->speed;
}

/* Sets behavior_boomerang component info. */
void
component_behavior_boomerang_init
(
 	ComponentBehaviorBoomerang* self, 
	f32 initialSpeed,
	f32 speed,
	u32 timer,
	u32 targetID
)
{
	self->speed = speed;
	self->initialSpeed = initialSpeed;
	self->timerMax = timer;
	self->timer = self->timerMax;
	self->targetID = targetID;

	printf("%f\n", self->initialSpeed);
}

/* Ticks behavior_boomerang component. */
void
component_behavior_boomerang_tick(ComponentBehaviorBoomerang* self, ECS* ecs)
{	
	if (self->isBoomerang)
	{

		if (!self->isBegin)
		{
			_begin(self, ecs);
			return;
		}

		_boomerang(self, ecs);
	}
	else
	{
		if (self->timer > 0)
		{
			self->timer--;

			if (self->timer <= 0)
			{
				self->timer = 0;
				self->isBoomerang = true;
			}
		}
	}
}
