#pragma once

#include "../../engine/file.h"
#include "../../engine/shader.h"
#include "../../engine/font.h"
#include "../../engine/texture.h"
#include "../../engine/sound.h"
#include "../../engine/music.h"

#define SHADER_COUNT SHADER_COLOR_QUAD + 1
typedef enum ShaderType
{
	SHADER_TEXTURE_QUAD,
	SHADER_COLOR_QUAD
} ShaderType;

typedef struct ShaderPaths
{
	const char* vertex;
	const char* fragment;
} ShaderPaths;

static const ShaderPaths SHADER_PATHS[SHADER_COUNT] =
{
	{
		.vertex = "res/shader/texture_quad.vs",
		.fragment = "res/shader/texture_quad.fs"
	},
	{
		.vertex = "res/shader/color_quad.vs",
		.fragment = "res/shader/color_quad.fs"
	}
};

#define TEXTURE_COUNT TEXTURE_GOODIES + 1
typedef enum TextureType
{
	TEXTURE_SHADOW,
	TEXTURE_PLAYER,
	TEXTURE_ELEMENTAL,
	TEXTURE_CUPPER,
	TEXTURE_CHIPPER,
	TEXTURE_CRUMBLER,
	TEXTURE_CUSTARPEDO,
	TEXTURE_EXCORSANT,
	TEXTURE_ROCK_CANDY_MICHEAL,
	TEXTURE_NEAPOLITOTEM,
	TEXTURE_EZ_ACHE,
	TEXTURE_CREPER,
	TEXTURE_GUM_MK_II,
	TEXTURE_SNAKE,
	TEXTURE_CHIP,
	TEXTURE_GUMBALL,
	TEXTURE_GOODIES
} TextureType;

static const char* TEXTURE_PATHS[TEXTURE_COUNT] =
{
	"res/gfx/play/shadow.png",
	"res/gfx/play/player.png",
	"res/gfx/play/elemental.png",
	"res/gfx/play/cupper.png",
	"res/gfx/play/chipper.png",
	"res/gfx/play/crumbler.png",
	"res/gfx/play/custarpedo.png",
	"res/gfx/play/excorsant.png",
	"res/gfx/play/rock_candy_micheal.png",
	"res/gfx/play/neapolitotem.png",
	"res/gfx/play/ez_ache.png",
	"res/gfx/play/creper.png",
	"res/gfx/play/gum_mk_ii.png",
	"res/gfx/play/snake.png",
	"res/gfx/play/chip.png",
	"res/gfx/play/gumball.png",
	"res/gfx/play/goodies.png"
};

#define FONT_COUNT FONT_UPHEAVAL + 1
typedef enum FontType
{
	FONT_UPHEAVAL
} FontType;

static const char* FONT_PATHS[FONT_COUNT] =
{
	"res/font/upheaval.ttf"
};

static const u32 FONT_SIZES[FONT_COUNT] =
{
	12
};

#define SOUND_COUNT SOUND_TEST + 1
typedef enum SoundType
{
	SOUND_TEST
} SoundType;

static const char* SOUND_PATHS[SOUND_COUNT] =
{
	"res/sound/test.ogg"
};

#define MUSIC_COUNT MUSIC_TEST + 1
typedef enum MusicType
{
	MUSIC_TEST
} MusicType;

static const char* MUSIC_PATHS[MUSIC_COUNT] =
{
	"res/music/test.ogg"
};

typedef struct Game Game;

typedef struct Resources
{
	Shader shaders[SHADER_COUNT];
	Texture textures[TEXTURE_COUNT];
	Music music[MUSIC_COUNT];
	Sound sounds[SOUND_COUNT]; 
	Font fonts[FONT_COUNT];
} Resources;
